final int screenWidth = 500;
final int screenHeight = 400;
final boolean DEBUG = false;
float DOWN_FORCE = 3.5;
float ACCELERATION = 1.3;
float DAMPENING = 0.75;

Rudi rudi;

void initialize() {
  frameRate(30);
  reset();
}

void reset(){
	clearScreens();
	addScreen("PlayerLevel", new PlayerLevel(4*width, height));
}

//----------------------------------------------------------
// LEVEL
//----------------------------------------------------------

class PlayerLevel extends Level {
  PlayerLevel(float levelWidth, float levelHeight) {
    super(levelWidth, levelHeight);
    setViewBox(0,0,screenWidth,screenHeight);
    addLevelLayer("background", new BackgroundLayer(this));
	addLevelLayer("playerlayer", new PlayerLayer(this));
	addLevelLayer("foreground", new ForegroundLayer(this));
  }
}

//----------------------------------------------------------
// LAYER
//----------------------------------------------------------

class PlayerLayer extends LevelLayer {
  PlayerLayer(Level owner) {
    super(owner);
	//Set Rudis Playerposition
    rudi = new Rudi(50, height-80);
	Skeleton skeleton = new Skeleton(200, height-120);
    addInteractor(skeleton);
	
	//Box so that the Player cant escape
	addBoundary(new Boundary(0,height-50,width,height-50));
	addBoundary(new Boundary(-1,0, -1,height));
	addBoundary(new Boundary(width+1,height, width+1,0));
	addBoundary(new Boundary(width+1, 0, -1,0));
	showBoundaries = DEBUG;
	//Add a player
    addPlayer(rudi);

	//-------------------------------------------------------
	
	//Start
	addGround(100, height-95, 200, height);
	
	//Jumps
	addGround(700, height-180, 740, height);
	addGround(800, height-220, 840, height);
	addGround(910, height-160, 950, height);
	
	//Default Ground
	addGround(-32, height-50, width+32, height);

	//Stairs
	addGround(500, height-95, 550, height);
	addGround(558, height-145, 654, height);
	
	//Cave
	addGround(1020, height-160, 1250, height);
	addGround(1100,-32, 1400, height-250);
	addGround(900,-32, 1100, 50);
	addGround(1300,-32, 1400, height-120);
	
	//Labyrinth
	//L1
	addGround(1500, height-95, 1900, height-95);
	addGround(1950, height-95, 2000, height-95);
	//L2
	addGround(1400, height-140, 1800, height-140);
	addGround(1900, height-140, 2000, height-140);
	//L3
	addGround(1500, height-185, 1900, height-185);
	addGround(1900, height-185, 2000, height-185);
	//L4
	addGround(1400, -32, width, height-250);
	//Vert
	addGround(1850, height-185, 1860, height-115);
	
	//-------------------------------------------------------
	
	//Start
	addBoundary(new Boundary(100,height-95, 200+10, height-95));
	
	//Stairs
	addBoundary(new Boundary(500, height-95, 550+10, height-95));
	addBoundary(new Boundary(500, height-50, 500, height-95));
	addBoundary(new Boundary(558,height-145, 654, height-145));
	addBoundary(new Boundary(558, height-95, 558, height-145));
	addBoundary(new Boundary(654, height-145, 654, height-50));
	
	//Jumps
	addBoundary(new Boundary(700, height-180, 740+15, height-180));
	addBoundary(new Boundary(800, height-220, 840+7, height-220));
	addBoundary(new Boundary(910, height-160, 950+15, height-160));

	//Cave
	addBoundary(new Boundary(1020, height-160, 1250+15, height-160));
	addBoundary(new Boundary(1020, height-50, 1020, height-160));
	addBoundary(new Boundary(1260, height-160, 1260, height-50));
	addBoundary(new Boundary(1300, height-225, 1100, height-225));
	addBoundary(new Boundary(1300, height-100, 1300, height-225));
	addBoundary(new Boundary(1400+10, height-95, 1300, height-95));
	addBoundary(new Boundary(1410,-32, 1410, height-100));
	
	//Labyrinth
	//L1 - large
	addBoundary(new Boundary(1500, height-95, 1900, height-95));
	addBoundary(new Boundary(1900, height-95, 1500, height-95));
	//L1 - small
	addBoundary(new Boundary(1950, height-95, 2000, height-95));
	addBoundary(new Boundary(2000, height-95, 1950, height-95));
	//L2 - large
	addBoundary(new Boundary(1400, height-140, 1800, height-140));
	addBoundary(new Boundary(1800, height-140, 1400, height-140));
	//l2 - small
	addBoundary(new Boundary(1900, height-140, 2000, height-140));
	addBoundary(new Boundary(2000, height-140, 1900, height-140));
	//L3 - large
	addBoundary(new Boundary(1500, height-185, 1900, height-185));
	addBoundary(new Boundary(1900, height-185, 1500, height-185));
	//L3 - small
	addBoundary(new Boundary(1900, height-185, 2000, height-185));
	addBoundary(new Boundary(2000, height-185, 1900, height-185));
	//L4 - large
	addBoundary(new Boundary(width, height-230, 1400, height-230));
	//Vert
	addBoundary(new Boundary(1850, height-95, 1850, height-170));
	addBoundary(new Boundary(1866, height-170, 1866, height-95));
	
	//-------------------------------------------------------
	
	//Add Triggers
	showTriggers = DEBUG;
	//Start
	addTrigger(new SkeletonTrigger(60, 0, 5, height, 210, height-80));
	addTrigger(new SkeletonTrigger(100, 0, 5, height, 210, height-80));
	addTrigger(new SkeletonTrigger(140, 0, 5, height, 210, height-80));
	//Stairs
	addTrigger(new SkeletonTrigger(300, 0, 5, height, 300, height-165));
	//Jumps
	addTrigger(new SkeletonTrigger(500, 0, 5, height, 250, height-80));
	//Cave
	addTrigger(new SkeletonTrigger(950, 0, 5, height, 250, height-200));
	//Labyrinth
	//L0
	addTrigger(new SkeletonTrigger(1300, 0, 5, height, 250, height-105));
	addTrigger(new SkeletonTrigger(1300, 0, 5, height, 450, height-105));
	addTrigger(new SkeletonTrigger(1300, 0, 5, height, 550, height-105));
	addTrigger(new SkeletonTrigger(1300, 0, 5, height, 650, height-105));
	//L1 - large
	addTrigger(new SkeletonTrigger(1300, 0, 5, height, 370, height-140));
	addTrigger(new SkeletonTrigger(1300, 0, 5, height, 470, height-140));
	//L2 - large
	addTrigger(new SkeletonTrigger(1300, 0, 5, height, 390, height-200));
	addTrigger(new SkeletonTrigger(1300, 0, 5, height, 490, height-200));
	addTrigger(new SkeletonTrigger(1300, 0, 5, height, 690, height-200));
	
	//-------------------------------------------------------
	
	//Add Exit
	addExit(width, height-185);
  }
  
  void draw() {
    super.draw();
    viewbox.track(parent, rudi);
  }
  
  void addGround(float a, float b, float c, float d){
	Sprite top = new Sprite("assets/ground/top.gif");
	TilingSprite groundtop = new TilingSprite(top, a, b, c, d+16);
	addBackgroundSprite(groundtop);
	
	Sprite filler = new Sprite("assets/ground/filler.gif");
	TilingSprite groundfiller = new TilingSprite(filler, a, b+16, c, d);
	addBackgroundSprite(groundfiller);
  }
  
  void addExit(float xpos, float hpos){
	Sprite exit = new Sprite("assets/background/exit.gif");
	exit.align(CENTER, BOTTOM);
	exit.setPosition(xpos, hpos);
	addForegroundSprite(exit);
	addForPlayerOnly(new ExitPoint(xpos,hpos-16));
  }

}

class BackgroundLayer extends LevelLayer {
	BackgroundLayer(Level owner){
		super(owner);
		addBackgroundSprite(new TilingSprite(new Sprite("assets/background/cave.gif"),0,0,width,height));
		//Ceiling
		addGround(-32, -32, 200, 50);
		addGround(208, -32, 300, 10);
		addGround(350, -32, 410, 20);
		addGround(414, -32, 512, 60);
	}
	void addGround(float a, float b, float c, float d){
	Sprite top = new Sprite("assets/ground/top.gif");
	TilingSprite groundtop = new TilingSprite(top, a, b, c, d+16);
	addBackgroundSprite(groundtop);
	
	Sprite filler = new Sprite("assets/ground/filler.gif");
	TilingSprite groundfiller = new TilingSprite(filler, a, b+16, c, d);
	addBackgroundSprite(groundfiller);
  }
}

class ForegroundLayer extends LevelLayer {
	ForegroundLayer(Level owner){
		super(owner);
		addGround(300, height-150, 400, height);
	}
	
	void addGround(float a, float b, float c, float d){
	Sprite top = new Sprite("assets/ground/top.gif");
	TilingSprite groundtop = new TilingSprite(top, a, b, c, d+16);
	addBackgroundSprite(groundtop);
	
	Sprite filler = new Sprite("assets/ground/filler.gif");
	TilingSprite groundfiller = new TilingSprite(filler, a, b+16, c, d);
	addBackgroundSprite(groundfiller);
  }
}

//----------------------------------------------------------
// RUDI
//----------------------------------------------------------

class Rudi extends Player {
	float speed = 2;
	Rudi(float x, float y) {
		super("Rudi");
		setupStates();
		setPosition(x,y);
		handleKey('W');
		handleKey('A');
		handleKey('D');
		handleKey('R');
		setForces(0,DOWN_FORCE);
		setAcceleration(0,ACCELERATION);
		setImpulseCoefficients(DAMPENING,DAMPENING);
	}
  
	void pickedUp(Pickup pickup) {
		if(pickup.name == "Exit Point"){
			setCurrentState("won");
		}
	}

	void setupStates() {
		addState(new State("idle", "assets/rudi/standing.gif"));
		addState(new State("running", "assets/rudi/running.gif", 1, 3));
	   
		State jumping = new State("jumping", "assets/rudi/jumping.gif");
		jumping.setDuration(10);
		addState(jumping);
		
		State dead = new State("dead", "assets/rudi/dead.gif",1,1);
		dead.setAnimationSpeed(0.25);
		dead.setDuration(15);
		addState(dead);
		
		State won = new State("won", "assets/rudi/standing.gif");
		won.setDuration(15);
		addState(won);

		setCurrentState("idle");  
	}
  
	void handleInput() {
		if(active.name=="dead") return;    
		
		if(isKeyDown('R')){
			reset();
		}
		
		if(isKeyDown('A') || isKeyDown('D')) {
		  if (isKeyDown('A')) {
			setHorizontalFlip(true);
			addImpulse(-speed, 0);
		  }
		  if (isKeyDown('D')) {
			setHorizontalFlip(false);
			addImpulse(speed, 0);
		  }
		}

		if(isKeyDown('W') && active.name!="jumping" && boundaries.size()>0) {
		  addImpulse(0,-35);
		  setCurrentState("jumping");
		}
		
		if (active.mayChange()) {
		  if(isKeyDown('A') || isKeyDown('D')) {
			setCurrentState("running");
		  }
		  else { setCurrentState("idle"); }
		}
	  }  

	void overlapOccurredWith(Actor other, float[] direction) {
		if (other instanceof Skeleton) {
		  Skeleton skeleton = (Skeleton) other;
		  float angle = direction[2];

		  float tolerance = radians(75);
		  if (PI/2 - tolerance <= angle && angle <= PI/2 + tolerance) {
			skeleton.squish();
			stop(0,0);
			setImpulse(0, -30);
			setCurrentState("jumping");
		  }

		  else { die(); }
		}
	}
	
	void die(){
		setCurrentState("dead");
		setInteracting(false);
		addImpulse(0,-20);
		setForces(0,20);
	}
	
	void handleStateFinished(State which){
		if(which.name == "dead" || which.name == "won"){
			removeActor();
			reset();
		}else{
			setCurrentState("idle");
		}
	} 
}

//----------------------------------------------------------
// SKELETON
//----------------------------------------------------------

class Skeleton extends Interactor {

  Skeleton(float x, float y) {
    super("Skeleton");
    setStates();
    setForces(-0.25, DOWN_FORCE);    
    setImpulseCoefficients(DAMPENING, DAMPENING);
    setPosition(x,y);
  }
  
  void setStates() {
    State walking = new State("idle", "assets/skeleton/walking.gif", 1, 3);
    walking.setAnimationSpeed(0.12);
    addState(walking);

    State naked = new State("naked", "assets/skeleton/naked.gif", 1, 3);
    naked.setAnimationSpeed(0.12);
    addState(naked);
	
	State dead = new State("dead", "assets/skeleton/dead.gif");
    dead.setAnimationSpeed(0.12);
    addState(dead);
    
    setCurrentState("idle");
  }
  
  void gotBlocked(Boundary b, float[] intersection) {
    if (b.x==b.xw) {
      fx = -fx;
      setHorizontalFlip(fx > 0);
    }
  }
  
  void squish() {
    if (active.name == "idle") {
      setCurrentState("naked");
      return;
    }
	if (active.name == "naked") {
      setCurrentState("dead");
      return;
    }
	if (active.name == "dead") {
      removeActor();
      return;
    }
  }
}

class SkeletonTrigger extends Trigger {
	float skeletonx, skeletony;
	SkeletonTrigger(float x, float y, float w, float h, float _sx, float _sy) {
		super("skeleton", x, y, w, h);
		skeletonx = _sx;
		skeletony = _sy;
	}

	void run(LevelLayer layer, Actor actor, float[] intersection) {
		Skeleton s = new Skeleton(x+skeletonx, skeletony);
		layer.addInteractor(s);
		removeTrigger();
	}
}

//----------------------------------------------------------
// PICKUP
//----------------------------------------------------------
class GamePickup extends Pickup {
	GamePickup(String name, String spritesheet, int rows, int columns, float x, float y, boolean visible){
		super(name, spritesheet, rows, columns, x, y, visible);
	}
}
//----------------------------------------------------------
// EXITPOINT
//----------------------------------------------------------
class ExitPoint extends GamePickup {
	ExitPoint(float x, float y) {
		super("Exit Point", "assets/background/torch.gif", 1, 1, x, y, true);
		Sprite exitp = getState("Exit Point").sprite;
		exitp.align(RIGHT, CENTER);
	}
}